import { Markup, MarkupView } from "models/widgets/markup";
import * as p from "core/properties";
import { div } from "core/dom";
import { ImportedStyleSheet, StyleSheetLike } from "core/dom";
declare var jsonpatch: any;
declare var JSONEditor: any;

export class bokehJSONEditorView extends MarkupView {
    declare model: bokehJSONEditor;

    override stylesheets(): StyleSheetLike[] {
        return [
            ...super.stylesheets(),
            new ImportedStyleSheet("/static/css/jsoneditor.css"),
        ];
    }

    private editor: any; // Assuming JSONEditor is correctly loaded

    private generatePatch(oldJson: any, newJson: any): any {
        // Use fast-json-patch to generate the patch
        const patch = jsonpatch.compare(oldJson, newJson);
    return patch;
}
    override connect_signals(): void {
    super.connect_signals();
    }

override render(): void {
    super.render();
    const container = div({
        class: 'json-editor-container',
        style: {
            width: `${this.model.width}px`,
            height: `${this.model.height}px`
        }
    });
    this.markup_el.appendChild(container);
    // Determine the mode based on the editable property
    const editorMode = this.model.editable ? 'tree' : 'view';
    this.editor = new JSONEditor(container, {
        mode: editorMode, // Set the mode
        onChangeJSON: (JSONDoc:any) => {

            const updatedJson = JSONDoc;
            const oldJson = this.model.onChangePatch.updatedJson || JSON.parse(this.model.code as string); // Initialize oldJson with the initial JSON
            const patch = this.generatePatch(oldJson, updatedJson); // Generate the patch
            console.log("Sending patch to Python:", { patch: patch, updatedJson: updatedJson });
            this.model.onChangePatch = { patch: patch, updatedJson: updatedJson }; // Update the onChangePatch property
        }
    });
    Object.getPrototypeOf(this.editor.node).DEBOUNCE_INTERVAL = this.model.debounce_interval;
    this.editor.set(JSON.parse(this.model.code as string)); // Set the initial JSON
    this.model.onChangePatch.updatedJson = JSON.parse(this.model.code as string); // Initialize onChangePatch.updatedJson with the initial JSON
}
}

export namespace bokehJSONEditor {
    export type Attrs = p.AttrsOf<Props>;
    export type Props = Markup.Props & {
        code: p.Property<string>;
        debounce_interval: p.Property<number>;
        onChangePatch: p.Property<any>;
        editable: p.Property<boolean>;
    };
}

export interface bokehJSONEditor extends bokehJSONEditor.Attrs {}

export class bokehJSONEditor extends Markup {
    properties: bokehJSONEditor.Props;
    static init_bokehJSONEditor(): void {
        this.prototype.default_view = bokehJSONEditorView;
        this.define<bokehJSONEditor.Props>(({ String, Int, Any, Boolean }) => ({
            code: [String, ''],
            debounce_interval: [Int, 2000],
            onChangePatch: [Any, {}],
            editable: [Boolean, true],  // Default is true
        }));
    }
}

bokehJSONEditor.init_bokehJSONEditor();

