from bokeh.models import Button, TextAreaInput, PreText, Div
from bokeh.layouts import column, row,layout,grid
from bokeh.plotting import curdoc
from utilities import bokehJSONEditorWithTitle
import pkg_resources
from dccXMLJSONConv.dccConv import XMLToJson, JSONToXML
from requests import post
from lxml import etree
import json
import os

print("CWD is "+str(os.getcwd()))
try:
    config=json.load(open('config.json'))
except FileNotFoundError:
    try:
        config = json.load(open('../../config.json'))
    except FileNotFoundError:
        config = json.load(open('dcc_XMLJSONConvGUI/config.json'))


dccRestServer=config['dccRestServer']
print("""CONFIG""")
print(config)
print("""CONFIG""")
import logging

import socket  # for hostname
try:
    with open('./exampleData/JSONExample.json', 'r') as file:
        jsonExamplStr=file.read()
except FileNotFoundError:
    with open('dcc_XMLJSONConvGUI/exampleData/JSONExample.json', 'r') as file:
        jsonExamplStr=file.read()
try:
    with open('./exampleData/XMLExample.xml', 'r') as file:
        xmlExamplStr=file.read()
except FileNotFoundError:
    with open('dcc_XMLJSONConvGUI/exampleData/XMLExample.xml', 'r') as file:
        xmlExamplStr=file.read()

__version_info__ = ('0', '0', '1')
__version__ = '.'.join(__version_info__)

class Page():
    def __init__(self):
        self.sheetNames = []
        self.jsonStr=""
        self.jsonDict={}
        self.xmlStr=""
        self.caontactInfos={'eMail':"""<a href="mailto:"""+config['contact']['email']+"""?subject = DCC XML/JSON Rest API GUI&body = Message" target="_blank">" """+config['contact']['email']+"""</a>""",
                          'tel':"""<a href="https://uc.ptb.de/openscapeuc/?dial="""+str(config['contact']['tel'])+""" target="_blank">0049 531 592 """+str(config['contact']['tel'])+"""</a>""",
                          'repo':"""<a href=" """+str(config['contact']['repoUrl'])+""" "target="_blank">" """+str(config['contact']['repoUrl'])+"""</a>""",
                          'issues':"""<a href=" """+str(config['contact']['repoUrl'])+"""/-/issues/new" target="_blank"" >Here</a>"""}
        self.caontactInfos = Div(
            text="Gui Version: " + str(__version__) + "DCC Converter Version"+ str(pkg_resources.get_distribution("dccXMLJSONConv").version)+ " Support E-Mail:" + self.caontactInfos['eMail'] + " Tel: " +
                 self.caontactInfos['tel'] + " Repo URL:" + self.caontactInfos['repo'] + " Issue tracker:" +
                 self.caontactInfos['issues'], height=28)
        self.xmlText=TextAreaInput(value=xmlExamplStr, title="XML",max_length=1000000,rows=50,width=900,height=800)
        self.jsonEditor = bokehJSONEditorWithTitle(title="JSON", code=jsonExamplStr, width=900, height=800)
        self.jsonToXMLButton=Button(label="Convert JSON to XML", button_type="default")
        self.jsonToXMLButton.on_click(self.jsonTOXMLCB)
        self.xmlToJSONButton = Button(label="Convert XML to JSON", button_type="default")
        self.xmlToJSONButton.on_click(self.xmlTOJSONCB)#
        #TODO add download functionality driectly to the widgets!
        self.xmlDownloadButton = Button(label="Download XML", button_type="default")
        self.xmlDownloadButton.on_click(self.xmlDownloadCB)
        self.jsonDownloadButton = Button(label="Download JSON", button_type="default")
        self.jsonDownloadButton.on_click(self.jsonDownloadCB)
        self.commandTextFiled = PreText(text="""HTTP Command >>>""", width=1800, height=100)
        self.errorMSG = PreText(text="""Log >>>""",width=1800, height=100)
        self.page = grid([
            [self.caontactInfos],
            [self.xmlText,self.jsonEditor.widget],
            [row([self.xmlToJSONButton,self.xmlDownloadButton]),row([self.jsonToXMLButton,self.jsonDownloadButton])],
            [self.errorMSG]
        ])
        #self.page=column(self.caontactInfos,row(column(self.xmlText, self.xmlToJSONButton),column(self.jsonEditor.widget, self.jsonToXMLButton)),self.errorMSG)

    def jsonTOXMLCB(self):
        self.xmlToJSONButton.button_type = "default"
        self.jsonStr=self.jsonEditor.bokehJSONEditor.code
        try:
            self.jsonDict=json.loads(self.jsonStr)
        except Exception as e:
            self.errorMSG.text=+str(Exception)+' '+str(e)
            return
        try:
            response = post(dccRestServer+"json2dcc/",json= {"js":self.jsonDict}, proxies=config['proxies'])
            if response.ok:
                # filter the requested dcc/xml from the full response-object
                xml_txt = response.text
                print(xml_txt)
                self.xmlStr = xml_txt
                try:
                    x = etree.ElementTree(etree.fromstring(xml_txt))
                    etree.indent(x, space="    ")
                    pretty_xml = etree.tostring(x, pretty_print=True, encoding=str)
                    self.xmlText.value =  str(pretty_xml)
                except:
                    self.xmlText.value = str(xml_txt)
                self.jsonToXMLButton.button_type="success"
                self.errorMSG.text = """Log >>>"""
            else:
                # if it failed give some information, why
                print("invalid response: %s" % response.reason)
                print("invalid response: %s" % response.text)
                print(json.loads(response.text)['detail'])
                self.errorMSG.text="""ERROR >>>"""+json.loads(response.text)['detail']
                self.jsonToXMLButton.button_type = "danger"
        except Exception as e:
            if e.__class__.__name__ == "ConnectionError":
                self.errorMSG.text = """ERROR XML<-->JSON Rest Server not reachable used local conversion as backup"""
            else:
                self.errorMSG.text = """ERROR >>>""" + str(e)+" used local conversion as backup"
            #print(self.jsonStr)
            self.xmlStr = JSONToXML(self.jsonStr)
            self.xmlText.value = self.xmlStr

    def xmlTOJSONCB(self):
        self.jsonToXMLButton.button_type = "default"
        self.xmlStr=self.xmlText.value
        try:
            response = post(dccRestServer+"dcc2json/",json={'xml':self.xmlStr}, proxies=config['proxies'])
            if response.ok:
                # filter the requested dcc/xml from the full response-object
                json_txt = response.text
                self.xmlToJSONButton.button_type = "success"
                self.errorMSG.text="""Log >>>"""
                self.jsonStr = json_txt
                self.jsonEditor.bokehJSONEditor.code = json.loads(json_txt)
            else:
                # if it failed give some information, why
                print("invalid response: %s" % response.reason)
                print("invalid response: %s" % response.text)
                print(json.loads(response.text)['detail'])
                self.errorMSG.text="""ERROR >>>"""+json.loads(response.text)['detail']
                self.xmlToJSONButton.button_type = "danger"

        except Exception as e:
            if e.__class__.__name__ == "ConnectionError":
                self.errorMSG.text = """ERROR XML<-->JSON Rest Server not reachable used local conversion as backup"""
            else:
                self.errorMSG.text = """ERROR >>>""" + str(e)+" used local conversion as backup"
            json_txt = json.dumps(XMLToJson(self.xmlStr))

            self.jsonStr=json_txt
            self.jsonEditor.bokehJSONEditor.code = json.loads(json_txt)
        print("Conversion Done")

    def xmlDownloadCB(self):
        self.xmlDownloadButton.label="XML Download is not implemented yet"
        self.xmlDownloadButton.button_type="danger"
        #TODO wirte file Download Button Class to be resued in our scripts as well and add to an bokehUtils Repo/module

    def jsonDownloadCB(self):
        self.jsonDownloadButton.label="JSON Download is not implemented yet"
        self.jsonDownloadButton.button_type="danger"
        #TODO wirte file Download Button Class to be resued in our scripts as well
    def p_callback(self):
        # if self.busyCount == 0:
        #    text = """<h3 style="background-color:#1F77B4;">%s</h3>""" % datetime.now().strftime("%d. %b %Y  &nbsp&nbsp    %H:%M:%S")
        #    self.clock.text = text
        # else:
        #    text = """<h3 style="background-color:coral;">%s   BUSY! </h3>""" % datetime.now().strftime("%d. %b %Y  &nbsp&nbsp    %H:%M:%S")
        #    self.clock.text = text
        pass


def make_doc(doc):
    LOG_FORMAT = "%(levelname)s %(asctime)s - %(message)s"
    file_handler = logging.FileHandler(filename='./bokeh.log', mode='w')
    file_handler.setFormatter(logging.Formatter(LOG_FORMAT))
    logger = logging.getLogger(__name__)
    logger.addHandler(file_handler)
    logger.setLevel(logging.DEBUG)

    logger.info("This is bok dcc_XMLJSONConvGUI ...")
    hostname = socket.gethostname()
    try:
        IPAddr = socket.gethostbyname(hostname)
    except Exception as e:
        logger.warning("Maybe there is something wrong with your /etc/hosts \n" + e)
        IPAddr = "localhost"
    logger.info("Opening Bokeh application on http://" + str(hostname) + ":5008/dccxmljsonconvgui")
    logger.info("Opening Bokeh application on http://" + str(IPAddr) + ":5008/dccxmljsonconvgui")
    myPage = Page()
    doc.add_root(myPage.page)
    doc.title = "DCC XML/JSON Rest API GUI"
    #doc.add_periodic_callback(myPage.p_callback, 500)
    print("Done")



# Now the server part

from tornado.ioloop import IOLoop

from bokeh.application.handlers import FunctionHandler
from bokeh.application import Application
from bokeh.server.server import Server

from jinja2 import Environment, FileSystemLoader

def main():
    """Launch the server and connect to it.
    """
    print("Preparing dcc_XMLJSONConvGUI application.")
    io_loop = IOLoop.current()
    bokeh_app = Application(FunctionHandler(make_doc))

    server = Server({"": bokeh_app},
                    io_loop=io_loop,
                    http_server_kwargs={'max_buffer_size': 900000000},
                    websocket_max_message_size=500000000,
                    port=5008,
                    allow_websocket_origin=['*'],
                    prefix="dccxmljsonconvgui",
                    #template_path = os.path.join(os.path.dirname(__file__), "templates"),
                    #index = os.path.join(os.path.dirname(__file__), "templates", "index.html"),
                    static_path = os.path.join(os.path.dirname(__file__), "dccxmljsonconvgui/static"),
                    )
    server.start()
    hostname = socket.gethostname()
    try:
        IPAddr = socket.gethostbyname(hostname)
    except Exception as e:
        print("Maybe there is something wrong with your /etc/hosts \n" + e)
        IPAddr = "localhost"
    print("Opening Bokeh application on http://" + str(hostname) + ":5008/dccxmljsonconvgui")
    print("Opening Bokeh application on http://" + str(IPAddr) + ":5008/dccxmljsonconvgui")
    io_loop.add_callback(server.show, "/")
    io_loop.start()

# This is run when called as python-script (python this_here.py)
if __name__ == '__main__':
    print("Entering main function now main()")
    main()

# This is run when called by bokeh (bokeh serve --show --port 5003 this_here.py)
print("RUN as server calling make_doc(curdoc())")
make_doc(curdoc())
