import { Area, AreaView } from "./area";
import * as hittest from "../../core/hittest";
import * as p from "../../core/properties";
import { StepMode } from "../../core/enums";
import { flip_step_mode } from "../../core/util/flip_step_mode";
import { Selection } from "../selections/selection";
export class VAreaStepView extends AreaView {
    static __name__ = "VAreaStepView";
    _index_data(index) {
        const { min, max } = Math;
        for (let i = 0; i < this.data_size; i++) {
            const x = this._x[i];
            const y1 = this._y1[i];
            const y2 = this._y2[i];
            index.add_rect(x, min(y1, y2), x, max(y1, y2));
        }
    }
    _step_path(ctx, mode, sx, sy, from_i, to_i) {
        // Assume the path was already moved to the first point
        let prev_x = sx[from_i];
        let prev_y = sy[from_i];
        const idx_dir = from_i < to_i ? 1 : -1;
        for (let i = from_i + idx_dir; i != to_i; i += idx_dir) {
            switch (mode) {
                case "before":
                    ctx.lineTo(prev_x, sy[i]);
                    ctx.lineTo(sx[i], sy[i]);
                    break;
                case "after":
                    ctx.lineTo(sx[i], prev_y);
                    ctx.lineTo(sx[i], sy[i]);
                    break;
                case "center":
                    {
                        const mid_x = (prev_x + sx[i]) / 2;
                        ctx.lineTo(mid_x, prev_y);
                        ctx.lineTo(mid_x, sy[i]);
                        ctx.lineTo(sx[i], sy[i]);
                        break;
                    }
            }
            prev_x = sx[i];
            prev_y = sy[i];
        }
    }
    _render(ctx, _indices, data) {
        const { sx, sy1, sy2 } = data ?? this;
        const forward_mode = this.model.step_mode;
        const backward_mode = flip_step_mode(this.model.step_mode);
        ctx.beginPath();
        ctx.moveTo(sx[0], sy1[0]);
        this._step_path(ctx, forward_mode, sx, sy1, 0, sx.length);
        this._step_path(ctx, backward_mode, sx, sy2, sx.length, -1);
        ctx.closePath();
        this.visuals.fill.apply(ctx);
        this.visuals.hatch.apply(ctx);
    }
    scenterxy(i) {
        const scx = this.sx[i];
        const scy = (this.sy1[i] + this.sy2[i]) / 2;
        return [scx, scy];
    }
    _hit_point(geometry) {
        const { sx, sy1, sy2 } = this;
        for (let i = 0; i < this.data_size - 1; i++) {
            let px;
            let py;
            switch (this.model.step_mode) {
                case "before":
                    px = [sx[i], sx[i + 1], sx[i + 1], sx[i]];
                    py = [sy1[i + 1], sy1[i + 1], sy2[i + 1], sy2[i + 1]];
                    break;
                case "after":
                    px = [sx[i], sx[i + 1], sx[i + 1], sx[i]];
                    py = [sy1[i], sy1[i], sy2[i], sy2[i]];
                    break;
                case "center":
                    {
                        const mid_x = (sx[i] + sx[i + 1]) / 2;
                        px = [sx[i], mid_x, mid_x, sx[i + 1], sx[i + 1], mid_x, mid_x, sx[i]];
                        py = [sy1[i], sy1[i], sy1[i + 1], sy1[i + 1], sy2[i + 1], sy2[i + 1], sy2[i], sy2[i]];
                        break;
                    }
            }
            if (hittest.point_in_poly(geometry.sx, geometry.sy, px, py)) {
                const result = new Selection();
                result.add_to_selected_glyphs(this.model);
                result.view = this;
                result.line_indices = [i];
                return result;
            }
        }
        return new Selection();
    }
    _map_data() {
        this.sx = this.renderer.xscale.v_compute(this._x);
        this.sy1 = this.renderer.yscale.v_compute(this._y1);
        this.sy2 = this.renderer.yscale.v_compute(this._y2);
    }
}
export class VAreaStep extends Area {
    static __name__ = "VAreaStep";
    constructor(attrs) {
        super(attrs);
    }
    static {
        this.prototype.default_view = VAreaStepView;
        this.define(({}) => ({
            x: [p.XCoordinateSpec, { field: "x" }],
            y1: [p.YCoordinateSpec, { field: "y1" }],
            y2: [p.YCoordinateSpec, { field: "y2" }],
            step_mode: [StepMode, "before"],
        }));
    }
}
//# sourceMappingURL=varea_step.js.map