import { logger } from "../../core/logging";
import { SelectionManager } from "../../core/selection_manager";
import { Signal, Signal0 } from "../../core/signaling";
import { uniq } from "../../core/util/array";
import { is_NDArray } from "../../core/util/ndarray";
import { keys, values } from "../../core/util/object";
import { isArray } from "../../core/util/types";
import { SelectionPolicy, UnionRenderers } from "../selections/interaction_policy";
import { Selection } from "../selections/selection";
import { DataSource } from "./data_source";
export class ColumnarDataSource extends DataSource {
    static __name__ = "ColumnarDataSource";
    get_array(key) {
        let column = this.data[key];
        if (column == null)
            this.data[key] = column = [];
        else if (!isArray(column))
            this.data[key] = column = Array.from(column);
        return column;
    }
    _select;
    inspect;
    selection_manager = new SelectionManager(this);
    constructor(attrs) {
        super(attrs);
    }
    static {
        this.define(({ Ref }) => ({
            selection_policy: [Ref(SelectionPolicy), () => new UnionRenderers()],
        }));
        this.internal(({ AnyRef }) => ({
            inspected: [AnyRef(), () => new Selection()],
        }));
    }
    initialize() {
        super.initialize();
        this._select = new Signal0(this, "select");
        this.inspect = new Signal(this, "inspect");
    }
    get_column(name) {
        return name in this.data ? this.data[name] : null;
    }
    columns() {
        // return the column names in this data source
        return keys(this.data);
    }
    get_length(soft = true) {
        const lengths = uniq(values(this.data).map((v) => is_NDArray(v) ? v.shape[0] : v.length));
        switch (lengths.length) {
            case 0: {
                return null; // XXX: don't guess, treat on case-by-case basis
            }
            case 1: {
                return lengths[0];
            }
            default: {
                const msg = "data source has columns of inconsistent lengths";
                if (soft) {
                    logger.warn(msg);
                    return lengths.sort()[0];
                }
                else
                    throw new Error(msg);
            }
        }
    }
    get length() {
        return this.get_length() ?? 0;
    }
    clear() {
        const empty = {};
        for (const col of this.columns()) {
            empty[col] = new this.data[col].constructor(0);
        }
        this.data = empty;
    }
    stream(new_data, rollover, { sync } = {}) {
        this.stream_to(this.properties.data, new_data, rollover, { sync });
    }
    patch(patches, { sync } = {}) {
        this.patch_to(this.properties.data, patches, { sync });
    }
}
//# sourceMappingURL=columnar_data_source.js.map