import warnings

import numpy as np

from bokeh.core.properties import String,Int,Any,Bool
from bokeh.models import Markup,Div
from bokeh.layouts import column
import json
import jsonpatch
#for categoricalPlot
def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return array[idx]

#for categoricalPlot and file Input
def flatten_and_align_nested_dict(d):
    def recurse_nested_dict(current_dict, path, flattened_data):
        if not isinstance(current_dict, dict):
            flattened_data.append((path.copy(), current_dict))
            return
        for key, value in current_dict.items():
            path.append(key)
            recurse_nested_dict(value, path, flattened_data)
            path.pop()

    flattened_data = []
    recurse_nested_dict(d, [], flattened_data)

    max_depth = max(len(item[0]) for item in flattened_data)
    aligned_data = {f'row{i+1}': [] for i in range(max_depth)}
    aligned_data['active'] = []

    for path, value in flattened_data:
        for i in range(max_depth):
            aligned_data[f'row{i+1}'].append(path[i] if i < len(path) else '')
        aligned_data['active'].append(value)

    return aligned_data
def extract_renames_from_patch(patch, old_json):
    renames = []
    removes = {op['path']: op for op in patch if op['op'] == 'remove'}
    adds = {op['path']: op for op in patch if op['op'] == 'add'}

    for add_path, add_op in adds.items():
        # Check if the corresponding remove operation exists
        if add_path in removes:
            remove_op = removes[add_path]
            old_key = remove_op['path'].split('/')[-1]  # Extract the key from the path
            new_key = add_op['path'].split('/')[-1]     # Extract the key from the path
            path = remove_op['path'].rsplit('/', 1)[0]  # Get the common path without the key
            renames.append((path, old_key, new_key))
        else:
            # Look for a remove operation that might indicate a rename
            for remove_path, remove_op in removes.items():
                if remove_path.startswith(add_path.rsplit('/', 1)[0]):
                    old_key = remove_op['path'].split('/')[-1]  # Extract the key from the path
                    new_key = add_op['path'].split('/')[-1]     # Extract the key from the path
                    path = remove_op['path'].rsplit('/', 1)[0]  # Get the common path without the key
                    renames.append((path, old_key, new_key))
                    break  # Stop searching after finding a match

    return renames

class bokehJSONEditor(Markup):
    # ... other properties ...

    __javascript__ = [
        "/static/js/jsoneditor.js",  # assuming jsoneditor.js is correctly served
        "/static/js/fast-json-patch.min.js",  # Adjust this path according to your server setup
    ]
    __implementation__ = open('./bokehJSONEditor.ts', 'r').read()

    code = String(default="{}")
    debounce_interval = Int(default=1000)
    onChangePatch = Any(default={})
    editable = Bool(default=True, help="Enable or disable editing in the JSONEditor.")
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.on_change('code', self._code_changed)
        self.on_change('onChangePatch', self._on_change_patch)

    def _code_changed(self, attr, old, new):
        pass
        # This method will be called whenever 'code' changes
        #print(f"Code changed from {old} to {new}")


    def _on_change_patch(self, attr, old, new):
        # Safely get the 'patch' key from new. If 'patch' is not in new, default to an empty list
        patch = new.get('patch', [])
        updated_json = new.get('updatedJson', {})  # Similarly for 'updatedJson'

        # If there's no patch or updated JSON, return early
        if not patch and not updated_json:
            warnings.warn(f"No Patch or updated JSON Found!")
            return

        current_json = json.loads(self.code)  # Assuming self.code is the current JSON state

        # Apply the patch to the current_json
        try:
            if patch:
                patched_json = jsonpatch.apply_patch(current_json, patch)
                self.code = json.dumps(patched_json)  # Update the internal state
                #print(f"Patch applied: {patch}")
        except jsonpatch.JsonPatchException as e:
            warnings.warn(f"Failed to apply patch: {e}")
            if updated_json:
                pass
                self.code = json.dumps(updated_json)  # Update the internal state with the updated JSON
                #print(f"Compleate JSON aplliyed !!!!")
        #print(f"Current JSON: {self.code}")


class bokehJSONEditorWithTitle:

    def __init__(self, title, **kwargs):
        self.title = title
        self.bokehJSONEditor = bokehJSONEditor(**kwargs)
        self.widget = column(children=[Div(text=self.title), self.bokehJSONEditor])

# ... (rest of your Bokeh application code) ...
